#! /usr/bin/perl

# Takes in a metacall file and exports the gains/loss/complex coordinates
# Optionally correct for maq's bug that shifts all coordinates by 1kb

use strict;
use Getopt::Long;
use Set::IntSpan;

my $window = 1000;
my $bCorrect = 1;
my $bHelp = 0;

GetOptions("window" => \$window,
			"correct" => \$bCorrect,
			"help" => \$bHelp);

if($bHelp)
{
	print "extractCN.pl [OPTIONS] [FILE]\n";
	print "extractCN.pl - parse the cnd viterbie output FILE (or stdin) and export the gains/losses\n";
	print "	--help	Print this message and exit\n";
	print "	--window=INT	Use INT as the window size (the parameter that was given to maq cns2win)\n";
	print "	--correct	Enable correction for the off-by-one bug in maq cns2win\n";
	exit(0);
}

my %stateTranslation = ('X' => 'X', 
						'C2' => 'G', 
						'C3' => 'G', 
						'RC2' => 'G',
						'RC3' => 'G',
						'C0.5' => 'L', 
						'RC0.5' => 'L',
						'Z' => 'L', 
						'RZ' => 'L',
						'N' => 'N');

my @stateList = ("G", "L"); # We care about gains and losses
my $stateFilter = "[" . join("", @stateList) . "]";
my $t_factor = 1000000;
my $half_window = $window / 2;

my $spans;

# Read in the coords
while(<>)
{
	chomp;
	my ($chr, $mp, $hr, $d, $r, $vit_state) = split(/\t/);
	my $state = $stateTranslation{$vit_state};
	if($state =~ /$stateFilter/)
	{
		# Coordinates are given by the midpoint of the range, convert to the enclosing coords
		my $start = parseAndCorrectCoord($mp) - $half_window;
		my $end = parseAndCorrectCoord($mp) + $half_window;

		if(!defined($spans->{$chr}->{$state}))
		{
			$spans->{$chr}->{$state} = Set::IntSpan->new();
		}
		$spans->{$chr}->{$state} += "$start-$end";
	}
}

foreach my $chr (keys %{$spans})
{
	foreach my $s (@stateList)
	{
		next if(!defined($spans->{$chr}->{$s}));
		my @runs = split(/,/,$spans->{$chr}->{$s}->run_list);
		foreach my $r (@runs)
		{
			my ($start, $end) = split(/-/, $r);
			print "$chr\t$start\t$end\t$s\n";
		}
	}
}


sub parseAndCorrectCoord
{
	my ($c) = @_;
	#Translate 
	$c *= $t_factor;

	# Correct
	if($bCorrect)
	{
		$c -= $window;
	}

	return $c;
}



