//-----------------------------------------------
// Copyright 2009 Wellcome Trust Sanger Institute
// Written by Jared Simpson (js18@sanger.ac.uk)
// Released under the GPL license
//-----------------------------------------------
import tango.math.Math;
import tango.io.Stdout;

class Bin
{
	this(real s, real e)
	{
		count = 0;
		proportion = 0;
		start = s;
		end = e;
	}

	int count;
	real proportion;
	real start; // inclusive
	real end; // exclusive
}

class BinData
{
	this(real[] data, real binSize)
	{
		// Calculate the min and max values
		real minV = real.max;
		real maxV = real.min;		
		foreach(d; data)
		{
			if(d < minV)
			{
				minV = d;
			}

			if(d > maxV)
			{
				maxV = d;
			}
		}
		
		m_binSize = binSize;
		m_lowerLimit = minV;
		m_upperLimit = maxV;

		int numBins = rndint(ceil((m_upperLimit - m_lowerLimit) / m_binSize)) + 1;
		m_bins.length = numBins;
		for(int i = 0; i < numBins; ++i)
		{
			m_bins[i] = new Bin(i*m_binSize, (i+1)*m_binSize);
		}
	
		// Put the data into the bins
		foreach(d; data)
		{
			int idx = calculateBinIdx(d);
			m_bins[idx].count++;
		}

		// Calculate the frequency
		foreach(b; m_bins)
		{
			b.proportion = cast(real)b.count / data.length;
		}
	}

	// Get the proportion of data at a particular value
	real getProportion(real value)
	{
		return m_bins[calculateBinIdx(value)].proportion;
	}

	void print()
	{
		foreach(b; m_bins)
		{
			Stdout.format("{0:F6}\t{}\t{2:F12}\n", b.start, b.count, b.proportion); 
		}
	}

	int calculateBinIdx(real data)
	{
		real t = data/m_binSize;
		int idx = rndint(t);
		//Stdout.format("{0:F6} -> {}\n", data, idx);
		return idx;
	}

	real m_binSize;
	real m_lowerLimit;
	real m_upperLimit;
	Bin[] m_bins;
}
