//-----------------------------------------------
// Copyright 2009 Wellcome Trust Sanger Institute
// Written by Jared Simpson (js18@sanger.ac.uk)
// Released under the GPL license
//-----------------------------------------------
import tango.math.Math;
import tango.io.Stdout;
import cnD;
import CNVModel;
import Common;
import Alphabet;
import Distribution;
import JointCNVDistribution;

class StateSummary
{
	this(MarkovModel m, SymbolArray[State] stateSymbols)
	{
		m_model = m;

		real[] repeatDepth;
		real[] repeatHet;
		real[] repeatHit;

		// Compute the mean/sd/n for each state and each column
		// Then compute the same for all the repeat states combined
		foreach(state; m.getStateList())
		{
			char[] stateName = state.toString();
			// Extract the symbols for this state
			real[] depthArray;
			real[] hetArray;
			real[] hitArray;
			extractValueArrays(stateName, m_model, stateSymbols, depthArray, hetArray, hitArray);
			
			// Compute the states for this state
			m_stats[stateName][DEPTH_COL_NAME] = computeStats(depthArray);
			m_stats[stateName][HET_COL_NAME] = computeStats(hetArray);
			m_stats[stateName][AVGHIT_COL_NAME] = computeStats(hitArray);

			if(stateName[0..1] == REPEAT_PREFIX)
			{
				repeatDepth ~= depthArray;
				repeatHet ~= hetArray;
				repeatHit ~= hitArray;
			}
		}

		m_stats[SNAME_RPTCMN][DEPTH_COL_NAME] = computeStats(repeatDepth);
		m_stats[SNAME_RPTCMN][HET_COL_NAME] = computeStats(repeatHet);
		m_stats[SNAME_RPTCMN][AVGHIT_COL_NAME] = computeStats(repeatHit);
	}

	Statistics getStats(char[] state, char[] col)
	{
		return m_stats[state][col];
	}

	void print()
	{
		char[][] columns = [DEPTH_COL_NAME, HET_COL_NAME, AVGHIT_COL_NAME];
		//Stdout.formatln("State\t{}\t{}\t{}", columns[0], columns[1], columns[2]);
		foreach(sn, aa; m_stats)
		{
			Stdout(sn)("\t");

			foreach(cn; columns)
			{
				Statistics stats = aa[cn];
				Stdout.format("{0}\t{1:F6}\t{2:F6}\t", stats.numDataPoints, stats.mean, stats.sd);
			}
			Stdout.newline;
		}
	}

	MarkovModel m_model;
	Statistics[char[]][char[]] m_stats;
}

//
// Extract the value arrays from a StateSymbol classification for a state
//
void extractValueArrays(char[] stateName, MarkovModel model, SymbolArray[State] stateSymbols, 
						out real[] depthArray, out real[] hetArray, out real[] avgHitArray)
{
	State s = model.getState(stateName);
	SymbolArray sa = stateSymbols[s];
	splitSymbolArray(sa, depthArray, hetArray, avgHitArray);
	return;
}

//
// Split an array of symbols into n-arrays of its constituent parts
//
void splitSymbolArray(SymbolArray symbolArray, out real[] depthArray, out real[] hetArray, out real[] avgHitArray)
{
	depthArray.length = symbolArray.length();
	hetArray.length = symbolArray.length();
	avgHitArray.length = symbolArray.length();

	int idx = 0;
	foreach(s; symbolArray)
	{
		// Cast the symbol to the correct type
		JointSymbol js = cast(JointSymbol)s;
		depthArray[idx] = js.depthSymbol.value;
		hetArray[idx] = js.hetSymbol.value;
		avgHitArray[idx] = js.hitSymbol.value;
		++idx;
	}	
}

